<?php
require_once(dirname(__FILE__) . '/go.inc.php');

class mtgGoGenerator extends mtgGenerator
{
  function makeTargets(mtgMetaInfo $meta)
  {
    $targets = array();

    $codegen = mtg_conf("codegen", null);
    if(!$codegen)
      $codegen = new mtgGoCodegen();
    $codegen->setMetaInfo($meta);

    $refl = new ReflectionClass($codegen);
    $SHARED_DEPS = array(
      dirname(__FILE__) . '/go.inc.php',
      dirname(__FILE__) . '/go_tpl.inc.php',
      __FILE__,
      $refl->getFileName()
    );

    mtg_mkdir(mtg_conf('out-dir'));

    $units = $meta->getUnits();
    $files = array();
    foreach($units as $unit)
      $files = array_merge($files, mtg_get_file_deps($meta, $unit));
    $files = array_unique($files);

    $bundle = mtg_conf("bundle", null);
    if($bundle && $units)
    {
      $targets[] = mtg_new_bundle($bundle,
                                  array_merge($SHARED_DEPS, $files),
                                  array('gen_go_bundle', $meta, $codegen, $units));
    }

    return $targets;
  }
}

function gen_go_struct($OUT, array $DEPS, mtgCodegen $codegen, mtgMetaInfoUnit $unit)
{
  return $codegen->genUnit($unit);
}

function gen_go_bundle($OUT, array $DEPS, mtgMetaInfo $meta, mtgGoCodegen $codegen, array $units)
{
  $bundle = '';
  $units_src = '';
  $create_struct_by_crc28 = '';
  $create_struct_by_name = '';
  $create_rpc_by_code = '';

  foreach($units as $unit)
  {
    if($unit->object instanceof mtgMetaEnum ||
       $unit->object instanceof mtgMetaStruct ||
       $unit->object instanceof mtgMetaRPC
    )
      $units_src .= $codegen->genUnit($unit);
  }

  foreach($units as $unit)
  {
    if($unit->object instanceof mtgMetaRPC)
    {
      $rpc = $unit->object;
      $create_rpc_by_code .= "\n    case {$rpc->getCode()}: { return New{$rpc->getName()}(), nil; }";
      continue;
    }

    if($unit->object->hasToken('POD') ||
      $unit->object instanceof mtgMetaFunc ||
      $unit->object instanceof mtgMetaEnum
     )
      continue;

    $create_struct_by_crc28 .= "\n    case {$unit->object->getClassId()}: { return New{$unit->object->getName()}(), nil }";
    $create_struct_by_name  .= "\n    case \"{$unit->object->getName()}\": { return New{$unit->object->getName()}(), nil }";
  }

  $templater = new mtg_go_templater();
  $tpl = $templater->tpl_bundle();
  return mtg_fill_template($tpl, 
    array('%bundle%' => $bundle,
          '%units_src%' => $units_src,
          '%create_struct_by_name%' => $create_struct_by_name,
          '%create_struct_by_crc28%' => $create_struct_by_crc28,
          '%create_rpc_by_code%' => $create_rpc_by_code,
    ));
}
